@@ -42,18 +42,24 @@ rule("swig.base")
                     end
                 end
             else
-                if target:is_plat("windows") then
+                if target:is_plat("windows", "mingw") then
                     target:set("extension", ".pyd")
                 end
             end
         elseif moduletype == "lua" then
             target:set("prefixname", "")
-            if not target:is_plat("windows") then
+            if not target:is_plat("windows", "mingw")  then
                 target:set("extension", ".so")
             end
         elseif moduletype == "java" then
-            target:set("prefixname", "lib")
-            if not target:is_plat("windows") then
+            if target:is_plat("windows", "mingw") then
+                target:set("prefixname", "")
+                target:set("extension", ".dll")
+            elseif target:is_plat("macosx") then
+                target:set("prefixname", "lib")
+                target:set("extension", ".dylib")
+            elseif target:is_plat("linux") then
+                target:set("prefixname", "lib")
                 target:set("extension", ".so")
             end
         else
@@ -68,17 +74,20 @@ rule("swig.base")
                     if fileconfig then
                         scriptdir = fileconfig.scriptdir
                     end
-                    local scriptfile = path.join(target:autogendir(), "rules", "swig", path.basename(sourcefile))
+                    local autogenfiles
+                    local autogendir = path.join(target:autogendir(), "rules", "swig")
                     if moduletype == "python" then
-                        scriptfile = scriptfile .. ".py"
+                        autogenfiles = os.files(path.join(autogendir, "*.py"))
                     elseif moduletype == "lua" then
-                        scriptfile = scriptfile .. ".lua"
+                        autogenfiles = os.files(path.join(autogendir, "*.lua"))
                     elseif moduletype == "java" then
-                        scriptfile = scriptfile .. ".java"
+                        autogenfiles = os.files(path.join(autogendir, "*.java"))
                     end
-                    table.insert(scriptfiles, scriptfile)
-                    if scriptdir then
-                        target:add("installfiles", scriptfile, {prefixdir = scriptdir})
+                    if autogenfiles then
+                        table.join2(scriptfiles, autogenfiles)
+                        if scriptdir then
+                            target:add("installfiles", autogenfiles, {prefixdir = scriptdir})
+                        end
                     end
                 end
             end
